/*
	EASY-ROB Development V2.3

	2 * 12 user defined robot dynamics routines for position controller and
	dynamic robot model are supported.

	int dyn_cntrl_user_1..12 (void)
	int dyn_model_user_1..12 (void)

	The EASY-ROB base program "easyrobx.exe" provides more than 100 functions.

	Stefan Anton

	Copyright (c) 1996,1997,1998

	modul: dyn_user.c

	To build a new er_dyn.dll, just execute the do_dlls.bat batch file in
	directory .\er_dvlp\.
*/
#include "stdafx.h"

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <math.h>

#include "..\er_dvlp\er_dvlp.h"

#pragma warning(disable : 4244)     // float conversation
#pragma warning(disable : 4244)     // float conversation

DllExport int er_dyn(void)
{
	return(0);
}
/*********************************************************
		Example:
		dyn_cntrl_user_1() - Position Controller
		dyn_model_user_1() - Simple dynamic robotmodel
*********************************************************/
/*
	User defined Structur for Controller User_1
*/
typedef struct {
	frame T;
	float a;
} DYN_CNTRL_USR_1;

static int dyn_cntrl_ini_1(void **dyn_cntrl_usr_ptr)
{
	DYN_CNTRL_USR_1 *dyn_ptr;
	if (*dyn_cntrl_usr_ptr==NULL) {
		moni_msg("dyn_cntrl_ini_1() *dyn_cntrl_usr_ptr=NULL -> malloc");
		*dyn_cntrl_usr_ptr = (void *)malloc(sizeof(DYN_CNTRL_USR_1));
		dyn_ptr = (DYN_CNTRL_USR_1 *)*dyn_cntrl_usr_ptr;
		if (dyn_ptr==NULL) {
			moni_msg("Fehler bei malloc DYN_CNTRL_USR_1 in dyn_cntrl_ini_1");
			return(1);
		}
		/*intialize values*/
		dyn_ptr->a=1.0;
		rob_kin_frame_ident(&(dyn_ptr->T));
	}
	return(0);
}
DllExport int  dyn_cntrl_user_1(void **dyn_cntrl_usr_ptr)
/*
	void **dyn_cntrl_usr_ptr
	This pointer is NULL, if this procedure is called the first time.
	You have the possibility to allocate memory for your user structure.
	Function dyn_cntrl_ini_1(dyn_cntrl_usr_ptr) will do this for the
	structure DYN_CNTRL_USR_1, see above.
	If this procedure is called more then one time, EASY-ROB will give
	you back the pointer to your user defined structure.
	In this user structure you can define any data you want.
	!! Take care to initialize your data !!

	The position controller function is called if Dynamics is ON.
	Every position controller sample rate "inq_dt_cntrl()" the
	driving torque "inq_uc()" is calculated dependent on the nominal -
	and actual joint values, the servo gains, etc.
*/
{
	int i;
	DYN_CNTRL_USR_1 *dyn_ptr; /* pointer to your user defined structure */
	float	*dq = inq_dq(),
			*q_dyn = inq_q_dyn(),
			*q = inq_q_solut(),
			*v_solut = inq_v_solut(),
			*e1 = inq_cntrl_e1(),
			*kpp = inq_cntrl_kpp(),
			*kpv = inq_cntrl_kpv(),
			*kiv = inq_cntrl_kiv(),
			*aux1 = inq_cntrl_aux1(),
			*uc_kiv1 = inq_cntrl_uc_kiv1(),
			*N = inq_model_N(),
			*uc = inq_uc(),
			*v_dyn = inq_v_dyn(),
			dt_cntrl = inq_dt_cntrl();

	if (dyn_cntrl_ini_1(dyn_cntrl_usr_ptr)) return(1);
	dyn_ptr = (DYN_CNTRL_USR_1 *)*dyn_cntrl_usr_ptr;
	if (dyn_ptr->a==0) {;} //do nothing with it

	/* P -Regler */
	for (i=0;i<inq_num_dofs();i++) {
		float dv,vs = kpp[i]*(dq[i]=q[i]-q_dyn[i]);
		dv = vs - v_dyn[i] +
			  aux1[i]* v_solut[i];
		uc_kiv1[i] += kpv[i] * kiv[i]*dt_cntrl * dv;
		uc[i] = (kpv[i]*dv + uc_kiv1[i]) * N[i];
		e1[i] = dv;
	}
	return(0);
}
/*
	User defined Structur for Robot Model User_1
*/
typedef struct {
	frame TT;
	float aa;
} DYN_MODEL_USR_1;

static int dyn_model_ini_1(void **dyn_model_usr_ptr)
{
	DYN_MODEL_USR_1 *dyn_ptr;
	if (*dyn_model_usr_ptr==NULL) {
		moni_msg("dyn_model_ini_1() *dyn_model_usr_ptr=NULL -> malloc");
		*dyn_model_usr_ptr = (void *)malloc(sizeof(DYN_MODEL_USR_1));
		dyn_ptr = (DYN_MODEL_USR_1 *)*dyn_model_usr_ptr;
		if (dyn_ptr==NULL) {
			moni_msg("Fehler bei malloc DYN_MODEL_USR_1 in dyn_model_ini_1");
			return(1);
		}
		/*intialize values*/
		dyn_ptr->aa=1.0;
		rob_kin_frame_ident(&(dyn_ptr->TT));
	}
	return(0);
}
DllExport int  dyn_model_user_1(void **dyn_model_usr_ptr)
{
	int i;
	float *q_dyn = inq_q_dyn(),
			*v_dyn = inq_v_dyn(),
			*vz1_v = inq_model_vz1_v(),
			*vz1_a = inq_model_vz1_a(),
			*z1 = inq_model_z1(),
			*N = inq_model_N(),
			*uc = inq_uc(),
			*uc1 = inq_uc1(),
			dt_sys = inq_dt_sys();

	if (dyn_model_ini_1(dyn_model_usr_ptr)) return(1);

	for (i=0;i<inq_num_dofs();i++) {
		v_dyn[i] = vz1_a[i]*z1[i] + (vz1_v[i]/N[i])*(1-vz1_a[i])*uc1[i];
		q_dyn[i] += v_dyn[i]*dt_sys;
		uc1[i] = uc[i];
		z1[i] = v_dyn[i];
	}
	return(0);
}
/*********************************************************
		Templates:
		dyn_cntrl_user_2..12() - Position Controller
		dyn_model_user_2..12() - Simple dynamic robotmodel
*********************************************************/
DllExport int  dyn_cntrl_user_2(void **dyn_cntrl_usr_ptr)
{
	if (*dyn_cntrl_usr_ptr==NULL) {
		moni_msg("*dyn_cntrl_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_model_user_2(void **dyn_model_usr_ptr)
{
	if (*dyn_model_usr_ptr==NULL) {
		moni_msg("*dyn_model_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_cntrl_user_3(void **dyn_cntrl_usr_ptr)
{
	if (*dyn_cntrl_usr_ptr==NULL) {
		moni_msg("*dyn_cntrl_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_model_user_3(void **dyn_model_usr_ptr)
{
	if (*dyn_model_usr_ptr==NULL) {
		moni_msg("*dyn_model_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_cntrl_user_4(void **dyn_cntrl_usr_ptr)
{
	if (*dyn_cntrl_usr_ptr==NULL) {
		moni_msg("*dyn_cntrl_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_model_user_4(void **dyn_model_usr_ptr)
{
	if (*dyn_model_usr_ptr==NULL) {
		moni_msg("*dyn_model_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_cntrl_user_5(void **dyn_cntrl_usr_ptr)
{
	if (*dyn_cntrl_usr_ptr==NULL) {
		moni_msg("*dyn_cntrl_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_model_user_5(void **dyn_model_usr_ptr)
{
	if (*dyn_model_usr_ptr==NULL) {
		moni_msg("*dyn_model_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_cntrl_user_6(void **dyn_cntrl_usr_ptr)
{
	if (*dyn_cntrl_usr_ptr==NULL) {
		moni_msg("*dyn_cntrl_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_model_user_6(void **dyn_model_usr_ptr)
{
	if (*dyn_model_usr_ptr==NULL) {
		moni_msg("*dyn_model_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_cntrl_user_7(void **dyn_cntrl_usr_ptr)
{
	if (*dyn_cntrl_usr_ptr==NULL) {
		moni_msg("*dyn_cntrl_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_model_user_7(void **dyn_model_usr_ptr)
{
	if (*dyn_model_usr_ptr==NULL) {
		moni_msg("*dyn_model_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_cntrl_user_8(void **dyn_cntrl_usr_ptr)
{
	if (*dyn_cntrl_usr_ptr==NULL) {
		moni_msg("*dyn_cntrl_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_model_user_8(void **dyn_model_usr_ptr)
{
	if (*dyn_model_usr_ptr==NULL) {
		moni_msg("*dyn_model_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_cntrl_user_9(void **dyn_cntrl_usr_ptr)
{
	if (*dyn_cntrl_usr_ptr==NULL) {
		moni_msg("*dyn_cntrl_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_model_user_9(void **dyn_model_usr_ptr)
{
	if (*dyn_model_usr_ptr==NULL) {
		moni_msg("*dyn_model_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_cntrl_user_10(void **dyn_cntrl_usr_ptr)
{
	if (*dyn_cntrl_usr_ptr==NULL) {
		moni_msg("*dyn_cntrl_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_model_user_10(void **dyn_model_usr_ptr)
{
	if (*dyn_model_usr_ptr==NULL) {
		moni_msg("*dyn_model_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_cntrl_user_11(void **dyn_cntrl_usr_ptr)
{
	if (*dyn_cntrl_usr_ptr==NULL) {
		moni_msg("*dyn_cntrl_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_model_user_11(void **dyn_model_usr_ptr)
{
	if (*dyn_model_usr_ptr==NULL) {
		moni_msg("*dyn_model_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_cntrl_user_12(void **dyn_cntrl_usr_ptr)
{
	if (*dyn_cntrl_usr_ptr==NULL) {
		moni_msg("*dyn_cntrl_usr_ptr==NULL");
		return(1);
	}
	return(0);
}
DllExport int  dyn_model_user_12(void **dyn_model_usr_ptr)
{
	if (*dyn_model_usr_ptr==NULL) {
		moni_msg("*dyn_model_usr_ptr==NULL");
		return(1);
	}
	return(0);
}

